<?php
/*
 * Copyright (c) 2009, The members of Ouga! project
 * Etienne Deparis <etienne.deparis@umaneti.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *
 * modification, are permitted provided that the following conditions are
 * met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of the Ouga! project nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * File system function
 */

/**
 * Return an array with information on curent url
 * @return array
 * @param variables array The core variables application array.
 */
function get_canonic_url(&$variables){
	$path = '';
	if(is_array($variables['path'])){
		foreach($variables['path'] as $p){
			if($p != $variables['module']){
				$p = preg_replace("#\d+#", '%d', $p);
				$path .= $p.'/';
			}
		}
		$path = substr($path, 0, -1);
	}
	return $path;
}

/**
 * Return the base path of the ouga installation
 * @return string
 */
function base_path(){
	if(OUGA_CLEAN_URL) return OUGA_BASE_PATH;
	return OUGA_BASE_PATH.'index.php?u=';
}

/**
 * Return the element passed in argument by its index
 * @return mixed
 * @param index int[optional] Index of the element to get
 */
function get_arg($index = 0){
	global $variables;

	if(isset($variables['path'][$index])
		&& !empty($variables['path'][$index])){

		return $variables['path'][$index];
	}

	return false;
}

/**
 * Return the path to the theme directory
 * @return string
 * @param theme string[optional] Theme name
 */
function path_to_theme($theme = OUGA_THEME){
	if(OUGA_ERROR_LEVEL >= 2){
		global $variables;
		$variables['flash_messages']['ouga_logs'][]
			= 'Path to theme: ' . $theme;
	}

	if($url = _exists('themes/' . $theme))
		return OUGA_BASE_PATH . $url . '/';

	throw_error('fatal', $theme . ': Bad theme path call');
}

/**
 * Return the path to the module directory
 * @return string
 * @param module string Module name
 */
function path_to_module($module, $bp = true){
	if($bp)
		$bpath = OUGA_BASE_PATH;
	else
		$bpath = '';

	if($url = _exists('modules/' . $module))
		return $bpath . $url . '/';

	throw_error('fatal', $module . ': Bad module path call');
}

/**
 * Checks if a file or a module exists and return his path
 * @return string
 * @param pattern string The name of the file or module to check
 */
function _exists($pattern){
	if(file_exists($pattern)) return $pattern;
	else{
		$urls = array(
			'repository/' . $_SERVER['SERVER_NAME'] . '/modules/'
			. $pattern,
			'repository/' . $_SERVER['SERVER_NAME'] . '/themes/'
			. $pattern,
			'repository/' . $_SERVER['SERVER_NAME'] . '/' . $pattern,
			'repository/default/modules/' . $pattern,
			'repository/default/themes/' . $pattern,
			'repository/default/' . $pattern
		);
		foreach($urls as $u){
			if(file_exists($u)) return $u;
		}
	}
	return false;
}

/**
 * Return the path of a modele to include
 * @return string
 * @param modele string The name of the modele to include (String)
 */
function use_modele($modele){
	global $variables;

	if(OUGA_ERROR_LEVEL >= 2){
		if(is_array($modele)){
			$debug_msg = 'Use modele: (';

			foreach($modele as $m) $debug_msg .= $m . ', ';

			$variables['flash_messages']['ouga_logs'][]
				= substr($debug_msg,0,-2) . ')';
		}else{
			$variables['flash_messages']['ouga_logs'][]
				= 'Use modele: ' . $modele;
		}
	}

	if(is_array($modele)){
		foreach($modele as $m){
			if(!in_array($m, $variables['modeles_already_used'])){
				require_once path_to_module($m, false) . 'modele.php';
				$variables['modeles_already_used'][] = $m;
			}
		}
	}else{
		if(!in_array($modele, $variables['modeles_already_used'])){
			require_once path_to_module($modele, false) . 'modele.php';
			$variables['modeles_already_used'][] = $modele;
		}
	}
}

/**
 * Return the path to the core module file.
 * Useful to call a module when we want to load it.
 * @return string
 * @param module string Module name
 */
function get_module($module){
	if(OUGA_ERROR_LEVEL >= 2){
		global $variables;
		$variables['flash_messages']['ouga_logs'][]
			= 'Get module: ' . $module;
	}

	return path_to_module($module, false) . 'controller.php';
}

function component($componentName){
	global ${$componentName};
	return ${$componentName};
}

/**
 * Others
 *
 */

/**
 * Check the presence of hook.
 * @param variables array The core variables application array.
 * There is at the moment 4 level of hook (from 0 to 3)
 */
function check_hooks(&$variables){

	$variables['hooks'] = array();

	$logmsg = 'Checking hook: (';

	foreach($variables['modules_list'] as $m){
		if($url = _exists($m.'/hook.php')){
			require_once $url;
			$func = $m . '_hook_init';
			if(function_exists($func)) $func($variables);
			$logmsg .= $url . ', ';
		}
	}

	if(OUGA_ERROR_LEVEL >= 2){
		global $variables;
		$variables['flash_messages']['ouga_logs'][]
			= substr($logmsg, 0, -2) . ')';
	}
}

/**
 * Run hooks or discover them.
 * @param step int Different hook step
 * @param variables array The core variables application array.
 */
function run_hook($step, &$variables){
	if($step == 0) check_hooks($variables);

	if(!$variables['hook_readonly'] && isset($variables['hooks'][$step])){

		foreach($variables['hooks'][$step] as $s){
			if(OUGA_ERROR_LEVEL >= 2){
				$variables['flash_messages']['ouga_logs'][]
					= 'Run hook at step ' . $step . ': (' . $s . ')';

				$s($variables);

				$variables['flash_messages']['ouga_logs'][]
					= '(/' . $s . ')';
			}else $s($variables);
		}
	}
}

function strclean($string){
	$string = stripslashes(trim($string));
	$string = htmlentities($string,ENT_QUOTES,'UTF-8');
	$string = strip_tags($string);
	return $string;
}

function strdirty($string){
	return html_entity_decode($string,ENT_QUOTES,'UTF-8');
}

function strtoascii($string, $charset = 'utf-8'){
	$title_urlized = htmlentities($string, ENT_NOQUOTES, $charset);

	$title_urlized = preg_replace(
		'#\&([A-za-z])(?:acute|cedil|circ|grave|ring|tilde|uml)\;#',
		'\1',
		$title_urlized
	);
	$title_urlized = preg_replace(
		'#\&([A-za-z]{2})(?:lig)\;#',
		'\1',
		$title_urlized
	); // pour les ligatures e.g. '&oelig;'
	$title_urlized = preg_replace(
		'#\&[^;]+\;#',
		'',
		$title_urlized
	); // supprime les autres caractères
	$title_urlized = str_replace(' ', '_', $title_urlized);

	return $title_urlized;
}


/* The following function (session_delete) has been found on php.net
 * in the "session_destroy" documentation page.
 */
function session_delete(){
	// Unset all of the session variables.
	$_SESSION = array();

	// If it's desired to kill the session, also delete the session cookie.
	// Note: This will destroy the session, and not just the session data!
	if (ini_get('session.use_cookies')) {
		$params = session_get_cookie_params();
		setcookie(session_name(), '', time() - 42000,
			$params['path'], $params['domain'],
			$params['secure']);//, $params['httponly']
		//);
	}

	// Finally, destroy the session.
	session_destroy();
}

function cleanup_all(){
	global $boot_sequence;

	if(in_array('bdd', $boot_sequence)){
		if(bdd_is_activ())
			bdd_close();
	}
}
?>
