<?php
/*
 * Copyright (c) 2009, The members of Ouga! project
 * Etienne Deparis <etienne.deparis@umaneti.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 *
 * modification, are permitted provided that the following conditions are
 * met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of the Ouga! project nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 * Install the module called in $variables or directly in the
 * "module" parameter.
 * @return
 * @param variables array The core variables application array.
 * @param module string[optional] The name of the module to install.
 */
function install_module(&$variables, $module = ''){
	if ($module == '') $module = $variables['module'];

	$variables['flash_messages']['ouga_logs'][]
		= '(install module ' . $module . ')';

	// fichier d'installation du module
	if(is_file(path_to_module($module, false).'install.php')){
		$install_source = path_to_module($module, false).'install.php';
		require_once $install_source;
		$bddfunc = $module.'_dbmodel';

		if(function_exists($bddfunc)){
			$installstring = array();
			foreach($bddfunc() as $table => $data){
				$installstring[] = parse_query_array(
					array(
						'type' => 'create',
						'table' => $table,
						'fields' => $data['fields'],
						'primary_key' => $data['primary_key'],
					)
				);
			}
			bdd_query($installstring, true);
		}

		$function = $module.'_install';
		$function($variables);

		copy(
			$install_source,
			_exists('cache') . '/' . $module . '_install.php'
		);

		$variables['flash_messages']['ouga_logs'][]
			= 'cached install file';
	}
	$variables['modules_list'][] = $module;
	$variables['modules_just_setup'][] = $module;
	$variables['flash_messages']['ouga_logs'][]
		= '(/install module ' . $module . ')';
}

/**
 * Uninstall the module which name is passed in module parameter
 * @return
 * @param variables array The core variables application array.
 * @param module string The name of the module to uninstall.
 */
function uninstall_module(&$variables, $module){

	$variables['flash_messages']['ouga_logs'][]
		= '(uninstall module ' . $module . ')';

	// fichier d'installation du module
	if($url = _exists('cache/'.$module.'_install.php')){
		require_once $url;
		$function = $module.'_uninstall';
		$function($variables);
		unlink($url);
	}

	// suppresion du module de la liste des modules installés
	$newlist = array();
	foreach($variables['modules_list'] as $item){
		if($item != $module){
			$newlist[] = $item;
		}
	}
	$variables['modules_list'] = $newlist;

	$variables['modules_just_uninstalled'][] = $module;

	$variables['flash_messages']['ouga_logs'][]
		= '(/uninstall module ' . $module . ')';

}

function update_module_list(&$variables){
	bdd_query(
		"update {variables} set content = '"
		. serialize($variables['modules_list'])
		. "' where name = 'installed_modules'",
		true
	);
}

/**
 * Update a module
 * @return
 * @param variables array The core variables application array.
 */
function upgrade_module(&$variables){

}

// we look to update modules at inclusion.
$variables['current_file'][] = 'lib_modules';

$variables['modules_list'] = array();

// Liste de tous les modules activés sur le site
$modules = array_slice(scandir(_exists('modules')), 2);
// if permitted, add the defaults modules
if(OUGA_USE_DEFAULT){
	$modules = array_merge(
		$modules,
		array_slice(
			scandir('repository/default/modules'),
			2
		)
	);
}

if(!is_array($modules)){
	throw_error('fatal', 'Error during listing of modules');
}

if(OUGA_BUILD_STEP){

	$modquery = bdd_query(
		"select content from {variables} where name = 'installed_modules'"
	);

	if(!$modquery){
		// Ouga is not installed, so we have to do it.
		require_once 'lib_install.php';
		$variables['modules_list'] = array();
	}else{

		/* Ouga is correctly installed... */
		$modda = bdd_fetch($modquery);
		$variables['modules_list'] = unserialize($modda['content']);
	}

	// we see if there is module to install, uninstall or update...
	$flag = false;
	$variables['modules_just_setup'] = array();
	$variables['modules_just_uninstalled'] = array();
	foreach($modules as $mo){
		if(!in_array($mo, $variables['modules_list'])){
			if($info = _exists('modules/'.$mo.'/module.info')){
				$mod_conf = parse_ini_file($info);
				if($mod_conf['platform'] >= OUGA_VERSION){
					install_module($variables, $mo);
					$flag = true;
				}
			}
		}else{
			if($info = _exists('modules/'.$mo.'/module.info')){
				$mod_conf = parse_ini_file($info);
				if($mod_conf['platform'] < OUGA_VERSION){
					uninstall_module($variables, $mo);
					$flag = true;
				}
			}
		}
	}
	foreach($variables['modules_list'] as $mo){
		if(!in_array($mo, $modules)){
			uninstall_module($variables, $mo);
			$flag = true;
		}
	}

	if($flag) update_module_list($variables);
	if(OUGA_USE_ACL){
		component('acl')->discover_acl($variables);
	}
}
?>
